;###########################################################################
;### Autor   : Jörn Kierstein                                            ###
;### Erstellt: 04.01.2014                                                ###
;### IDE     : C64 Studio 5.3                                            ###
;###                                                                     ###
;### (c) 2014, 2018 by www.retro-programming.de                          ###
;###########################################################################

!to "landung.prg",cbm

;*** Konstanten einbinden
!source "..\_Includes_\system_const.asm"
!source "..\_Includes_\love_const.asm"

RASTERIRQ_SKY       = $00           ;ab hier die Himmelfarbe
RASTERIRQ_LASER     = $30-2         ;das 'Laser-Raster' 
                                    ;-2 für den stabilen Raster-IRQ
SPACESHIP_STARTX    = $a5           ;X-Position und
SPACESHIP_STARTY    = $32           ;Y-Position, an der das Schiff startet
GRAVITY             = $08           ;Anziehungskraft
THRUST_X            = $04           ;Schubkraft seitwärts
THRUST_Y            = $06           ;Schubkraft aufwärts


!zone BASIC_Start
;*** Startadresse
*=$0801

;*** BASIC-Zeile
 !word main-2
 !word 2018
 !text $9E," 2062",$00,$00,$00

!zone Main
;*******************************************************************************
;*** Einstieg ins Assemblerprogramm
;*******************************************************************************
main
;*** Start des Hauptprogramms
 lda #$00                           ;erstmal fest den 2. Port
 sta joystickInput+1                ;für den Joystick setzen

 jsr initLander                     ;Level starten 

;*** Raster-IRQ einrichten
 sei                                ;IRQs sperren
 lda #<landerIRQ                    ;Routine unserer Rasten-Routine
 sta JT_IRQVECTOR                   ;im den RAM-Vector speichern
 lda #>landerIRQ                    ;auch das MSB
 sta JT_IRQVECTOR+1
 lda VIC_IRQMASK                    ;Raster-IRQs auslösen soll
 ora #%00000001                     ;Dem VIC-II mitteilen, dass er 
 sta VIC_IRQMASK                    ;Raster-IRQs auslösen soll
 lda #RASTERIRQ_SKY                 ;Erster Raster-IRQ für den Himmel
 sta VIC_RASTERROWPOS                         
 lda VIC_CONTROLREG1                ;Höchstes BIT für die Rasterzeile
 and #%01111111                     ;löschen
 sta VIC_CONTROLREG1
 lda #%01111111                     ;Timer-IRQ 
 sta CIA1_IRQCONTROL                ;abschalten
 lda CIA1_IRQCONTROL                ;und zur Sicherheit bestätigen
 cli                                ;IRQs wieder erlauben

gameloop
 jmp gameloop                       ;Hauptschleife



!zone initLander
;*******************************************************************************
;*** Leveldaten zurücksetzen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
initLander
 lda #$00                           ;'Nachkommastellen' löschen
 sta Spaceship_PosX            
 sta Spaceship_PosY
 sta Spaceship_SpeedX               ;Geschwindigkeit löschen   
 sta Spaceship_SpeedX+1
 sta Spaceship_SpeedY
 sta Spaceship_SpeedY+1
 lda #SPACESHIP_STARTX              ;Startposition setzen
 sta Spaceship_PosX+1               ;X
 lda #SPACESHIP_STARTY              ;und
 sta Spaceship_PosY+1               ;Y
 jsr initSprites                    ;Sprites initialisieren
 jsr drawCanyon                     ;den Canyon zeichnen
 rts                                ;zurück



!zone initSprites
;*******************************************************************************
;*** Sprites initialisieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, SR
;*******************************************************************************
initSprites
 ldx #spriteSpaceship/64            ;Nr. des ersten 64-Byte-Sprite-Blocks
 stx SPRITEPOINTER0+4               ;Spritedaten zuweisen
 inx
 stx SPRITEPOINTER0+0
 inx
 stx SPRITEPOINTER0+2
 inx
 stx SPRITEPOINTER0+3
 inx
 stx SPRITEPOINTER0+1

 jsr updateSprites                  ;Sprite-Positionen setzen

 lda #COLOR_BLACK                   ;Farben der Sprites setzen
 sta VIC_SPRITE0COLOR

 lda #COLOR_LIGHTGREY
 sta VIC_SPRITE4COLOR

 lda #COLOR_RED
 sta VIC_SPRITEMULTICOLOR0
 lda #COLOR_YELLOW
 sta VIC_SPRITEMULTICOLOR1

 lda #%00001110                     ;nur die "Feuer"-Sprites sind MultiColor
 sta VIC_SPRITEMULTICOLOR

 lda #%00000000
 sta VIC_SPRITEDEEP
 sta VIC_SPRITEDOUBLEWIDTH
 sta VIC_SPRITEDOUBLEHEIGHT

 lda #%000111111                     ;erstmal nur das 'Schiff' anzeigen
 sta VIC_SPRITEACTIVE

 rts                                ;zurück



!zone updateSprites
;*******************************************************************************
;*** Sprites positionieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Sprites positionieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, SR
;*******************************************************************************
updateSprites
 lda Spaceship_SpeedX               ;LSB der X-Geschwindigkeit in den Akku
 clc                                ;C-Flag für Addition löschen
 adc Spaceship_PosX                 ;zum LSB der X-Pos. addieren
 sta Spaceship_PosX                 ;und speichern
 lda Spaceship_SpeedX+1             ;jetzt das MSB der X-Speed in den Akku
 adc Spaceship_PosX+1               ;zum MSB der X-Pos. addieren (C wird beachtet)
 sta Spaceship_PosX+1               ;und speichern
 tax                                ;X-Position (nur Vorkommateil) holen
 stx VIC_SPRITE0X                   ;für Sprite-0
 stx VIC_SPRITE1X                   ;1 und
 stx VIC_SPRITE4X                   ;4 übernehmen
 inx                                ;rechtes 'Feuer'
 inx                                ;um drei Pixel nach rechts
 inx                                ;verschieben
 stx VIC_SPRITE3X                   ;und positionieren

 tax                                ;nochmal die X-Position holen
 dex                                ;und linkes 'Feuer'
 dex                                ;um drei Pixel nach links
 dex                                ;verschieben
 stx VIC_SPRITE2X

 lda Spaceship_SpeedY               ;LSB der Y-Geschwindigkeit in den Akku
 clc                                ;Carry löschen
 adc Spaceship_PosY                 ;LSB der Y-Pos. addieren
 sta Spaceship_PosY                 ;und speichern
 lda Spaceship_SpeedY+1             ;MSB der Y-Geschwindigkeit in den Akku
 adc Spaceship_PosY+1               ;MSB der Y-Pos. addieren
 sta Spaceship_PosY+1               ;speichern und 
 sta VIC_SPRITE0Y                   ;für alle fünf Sprites übernehmen
 sta VIC_SPRITE1Y                
 sta VIC_SPRITE2Y                
 sta VIC_SPRITE3Y                
 sta VIC_SPRITE4Y         
 rts                                ;zurück



!zone drawCanyon
;*******************************************************************************
;*** Den Canyon zeichnen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
drawCanyon
 lda #COLOR_ORANGE
 sta VIC_BORDERCOLOR
 lda #COLOR_GREY
 sta VIC_BACKGROUNDCOLOR
 ;*** Canyon anzeigen
 ldx #<screenCanyon                 ;LSB des Canyonbildes ins X-Register
 ldy #>screenCanyon                 ;MSB des Canyonbildes ins Y-Register
 jsr drawScreen                     ;Bild ausgeben
 rts                                ;zurück


!zone joystickInput
;*******************************************************************************
;*** Den ausgewählten Joystick abfragen
;*******************************************************************************
;*** Übergabe: CIA1_<PORT> an joystickInput:+1
;*******************************************************************************
;*** Rückgabe: A = aktueller Status
;***           InputState = aktueller Status
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
joystickInput
 lda CIA1_A                        ;Aktuellen Status von Joy-1 oder 2 holen
                                   ;Nach der Joystick-Wahl speichern wir
                                   ;die richtige Adresse direkt hinterm lda
 sta InputState                    ;Aktuellen Status merken
 rts                               ;zurück



!zone checkInput
;*******************************************************************************
;*** Joystickeingabe prüfen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: Letzte Eingabe in InputState
;*******************************************************************************
;*** ändert  : A, X, Y, SR, InputState
;*******************************************************************************
checkInput
 lda VIC_SPRITEACTIVE               ;alle Flammen für die Düsen
 and #%11110001                     ;ausblenden und
 tax                                ;im X-Register merken

 jsr joystickInput                  ;Aktuellen Joystick-Status holen
 and #%00011111                     ;Nur die unteren 5-BIT sind von Interesse
 eor #%00011111                     ;Da Low-Aktiv, umkehren
 beq .noFire                        ;keine Eingabe, dann ENDE

 lsr                                ;rauf und
 lsr                                ;runter benötigen wir nicht
 lsr                                ;links?
 bcc .right                         ;falls nicht rechts prüfen
 tay                                ;Akku im Y-Reg. retten
 txa                                ;sichtbare Sprites (X-Reg.) in den Akku
 ora #%00001000                     ;Feuerschub rechts sichtbar
 tax                                ;wieder im X-Reg. merken
 sec                                ;C-Flag für Subtraktion setzen
 lda Spaceship_SpeedX               ;LSB der X-Geschwindigkeit in den Akku
 sbc #THRUST_X                      ;X-Beschleunigung abziehen
 sta Spaceship_SpeedX               ;speichern
 lda Spaceship_SpeedX+1             ;MSB der X-Geschwindigkeit holen
 sbc #$00                           ;ggf. C-Flag addieren
 sta Spaceship_SpeedX+1             ;und ebenfalls speichern
 tya                                ;Akku zurücksetzen
.right
 lsr                                ;rechts?
 bcc .fire                          ;falls nicht, Feuernknopf prüfen
 tay                                ;wieder Akku im Y-Reg. retten
 txa                                ;sichtbare Sprites holen
 ora #%00000100                     ;diesmal Feuerschub links aktivieren
 tax                                ;wieder im X-Reg. merken
 clc                            
 lda Spaceship_SpeedX               ;wie eben, nur dass jetzt
 adc #THRUST_X                      ;addiert wird
 sta Spaceship_SpeedX
 lda Spaceship_SpeedX+1
 adc #$00
 sta Spaceship_SpeedX+1
 tya                                ;Akku zurückholen
.fire
 lsr                                ;Feuerknopf gedrückt?
 bcc .noFire                        ;falls nicht Anziehungskraft addieren -> noFire:
 txa                                ;diesmal direkt X in den Akku holen
 ora #%00000010                     ;Feuerschub unten aktivieren
 tax                                ;wieder im X-Register merken
 sec                            
 lda Spaceship_SpeedY               ;kein nennenswerter Unterschied zur
 sbc #THRUST_Y                      ;links/rechts Beschleunigung
 sta Spaceship_SpeedY
 lda Spaceship_SpeedY+1
 sbc #$00
 sta Spaceship_SpeedY+1
.exit
                                    ;im X-Register stehen die sichtbaren 
 stx VIC_SPRITEACTIVE               ;Sprites, die wir jetzt wieder setzen
 rts                                ;zurück

.noFire                             ;Falls der Feuerknopf nicht gedrückt ist
 clc                                ;addieren wir die Gravitation
 lda Spaceship_SpeedY               ;dass funktioniert wie weiter oben
 adc #GRAVITY
 sta Spaceship_SpeedY
 lda Spaceship_SpeedY+1
 adc #$00
 sta Spaceship_SpeedY+1
 jmp .exit                          ;zum Ende springen



!zone landerIRQ
;*******************************************************************************
;*** Interrupthandler
;***
;*** Der 1. Rasterzeileninterrupt wird hier verarbeitet und setzt den
;*** RAM-Vector für den stabilen Raster-IRQ dann auf doRasterIRQ
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : -
;*******************************************************************************
landerIRQ
 lda #COLOR_LIGHTBLUE               ;hellblau in den Akku
 sta VIC_BORDERCOLOR                ;die Rahmenfarbe darauf setzen
 lda #RASTERIRQ_LASER               ;nächsten Raster-IRQ beim Laser
 sta VIC_RASTERROWPOS
 lda #<doRasterIRQ                  ;Adresse des Laser-IRQs in den
 sta JT_IRQVECTOR                   ;RAM-Vector
 lda #>doRasterIRQ                  ;auch das MSB
 sta JT_IRQVECTOR+1
 jsr updateSprites                  ;Sprites positionieren
 jmp .exit                          ;...und Interrupt verlassen
 
!align 255,0
doRasterIRQ
;*** Wenn wir hier landen, sind bereits 38-45 Taktzyklen
;*** in der aktuellen Rasterzeile (46 = $2e = $30-2) vergangen!

;*** Zweiten IRQ einrichten
;*** Da die Zeit bei aktivierten ROM nicht reicht,
;*** können wir den 2. Raster-IRQ erst in der übernächsten Zeile bekommen
 lda #<doubleIRQ                    ;(2 TZ) 2. Raster-IRQ einrichten
 sta JT_IRQVECTOR                   ;(4 TZ)
 lda #>doubleIRQ                    ;(2 TZ)
 sta JT_IRQVECTOR+1                 ;(4 TZ)
 tsx                                ;(2 TZ) Stackpointer im X-Reg. retten
 stx doubleIRQ+1                    ;(4 TZ) und fürs zurückholen sichern!
 nop                                ;(2 TZ)
 nop                                ;(2 TZ)
 nop                                ;(2 TZ)
 lda #%00000001                     ;(2 TZ) 1. Raster-IRQ später bestätigen
                                    ;------
                                    ;26 TZ

;*** Jetzt sind 64-71 Taktzyklen vergangen und wir sind
;*** auf jeden Fall in nächsten Rasterzeile (47)!
;*** Verbraucht wurden 1-8 TZ
 inc VIC_RASTERROWPOS               ;(6 TZ) 2. IRQ in der übernächsten Zeile (48)!!!
                                    ;       $d012 wurde bereits automatisch erhöht
 sta VIC_IRQSTATUS                  ;(4 TZ) IRQ bestätigen
 cli                                ;(2 TZ) Interrupts für den 2. Raster-IRQ
                                    ;       wieder freigeben

;*** Wir befinden uns in Rasterzeile 47 und
;*** haben bisher 13-20 Zyklen verbraucht

;*** etwas Zeit verschwenden...
 ldx #$08                           ;            2 TZ
.loop
 dex                                ;8 * 2 TZ = 16 TZ
 bne .loop                          ;7 * 3 TZ = 21 TZ
                                    ;1 * 2 TZ =  2 TZ
                                    ;          ------
                                    ;           41 TZ

;*** Bis hier sind 54-61 Taktzyklen vergangen, jetzt auf den IRQ warten...
;*** Der nächste Rasterinterrupt wird während dieser NOPs auftreten!
 nop                                ;2 TZ (56)
 nop                                ;2 TZ (58)
 nop                                ;2 TZ (60)
 nop                                ;2 TZ (62)
 nop                                ;2 TZ (64)
 nop                                ;2 TZ (66)

doubleIRQ
;*** Wir sind nun in Rasterzeile 48 und
;*** haben bisher genau 38 oder 39 Taktzyklen benötigt!!
;*** Wir können so sicher sein, da der IRQ während der NOPs auftrat.

;*** Jetzt exakt soviele Taktzyklen 'verschwenden', wie in
;*** dieser Zeile noch zu verarbeiten sind (also 24 oder 25).
 ldx #$00                           ;(2 TZ) Platzhalter für 1. Stackpointer
 txs                                ;(2 TZ) Stackpointer vom 1. IRQ wiederherstellen
 nop                                ;(2 TZ)
 nop                                ;(2 TZ)
 nop                                ;(2 TZ)
 bit $01                            ;(3 TZ)
 ldx VIC_RASTERROWPOS               ;(4 TZ)
 lda LaserFlash                     ;(4 TZ) 'Laserfarbe' schonmal in den Akku   
 cpx VIC_RASTERROWPOS               ;(4 TZ) sind wir noch in Rasterzeile 48?
                                    ;------
                                    ;25 TZ = 63 oder 64 TZ!!!

 beq .laser                         ;(3 TZ) wenn JA einen letzten Takt 'verschwenden'
                                    ;(2 TZ) sonst einfach weiterlaufen...

;*** Wir beginnen also immer exakt nach 3 TZ in der dritten Rasterzeile (49)
;*** nach dem 1. Raster-IRQ (den hatten wir ja in für Zeile 46 festgelegt)
.laser
 sta VIC_BORDERCOLOR                ;Rahmenfarbe setzen
 inc LaserFlash                     ;'Farbe' fürs Laser-Raster erhöhen

 ldx #$14                           ;Warteschleifen-Anzahl
 lda Spaceship_PosY+1               ;MSB der Y-Pos. des Raumschiffs in den Akku
 cmp #$32                           ;ist es in Zeile 50 (= $32)?
 bne .skip                          ;nein, dann direkt zur Warteschleife
 dex                                ;sonst, die Schleife etwas verkürzen
 dex
.skip
 dex                                ;kleine Warteschleife, damit der Laser
 bne *-1                            ;zwei Zeilen 'hoch' ist

 lda #COLOR_ORANGE                  ;hier beginnt auf jeden Fall der Canyon
 sta VIC_BORDERCOLOR                ;also Canyon-Farbe in den Rahmen

 lda #RASTERIRQ_SKY                 ;nächsten Raster-IRQ für den Himmel
 sta VIC_RASTERROWPOS

 lda #<landerIRQ                    ;und den RAM-Vector wieder 
 sta JT_IRQVECTOR                   ;auf die erste Routine setzen
 lda #>landerIRQ
 sta JT_IRQVECTOR+1
.exit
 jsr checkInput                     ;Joystick abfragen & verarbeiten
 lda #%00000001
 sta VIC_IRQSTATUS                  ;VIC-IRQ bestätigen
 pla                                ;Register zurückholen
 tay                                ;Y
 pla
 tax                                ;X
 pla                                ;Akku

 rti                                ;IRQ verlassen



;*******************************************************************************
;*** Allgemeine Funktionen einbinden
;*******************************************************************************
!source "..\_Includes_\love_lib.asm"



!zone Data
;*******************************************************************************
;*** Daten
;*******************************************************************************
LaserFlash
 !byte $00                          ;Rahmenfarbe fürs Laser-Raster

Spaceship_PosX
 !byte $00
 !byte SPACESHIP_STARTX              ;X- und  
Spaceship_PosY
 !byte $00
 !byte SPACESHIP_STARTY              ;Y-Position des Landungsschiffs

Spaceship_SpeedX
 !word $00
Spaceship_SpeedY
 !word $00
 
InputState
 !byte $00                          ;letzte Joystickeingabe



;*** Die Sprites
!align 63,0
spriteSpaceship
 !media "Ship.spriteproject",sprite,0,5

 
;*** Der Canyon
screenCanyon
 !media "Canyon.charscreen",charcolor