;###########################################################################
;### Autor   : Jörn Kierstein                                            ###
;### Erstellt: 01.12.2013                                                ###
;### IDE     : C64 Studio 5.3                                            ###
;###                                                                     ###
;### (c) 2013, 2018 by www.retro-programming.de                          ###
;###########################################################################

!to "puzzle.prg",cbm

;*** Konstanten einbinden
!source "..\_Includes_\system_const.asm"
!source "..\_Includes_\love_const.asm"



!zone BASIC_Start
;*** Startadresse
*=$0801

;*** BASIC-Zeile
 !word main-2
 !word 2018
 !text $9E," 2062",$00,$00,$00

!zone ASM_Start
;*******************************************************************************
;*** Einstieg ins Assemblerprogramm
;*******************************************************************************
main
 lda VIC_RASTERROWPOS               ;aktuelle Rasterzeile für rndSeed holen
 sta rndSeed                        ;und unter rndSeed speichern
 jsr showScreen_Title               ;Startbildschirm anzeigen
 jsr puzzleMain                     ;das Puzzle starten
 rts                                ;Zurück zum BASIC



;*******************************************************************************
;*** Das Puzzlespiel starten
;*******************************************************************************
!zone puzzleMain
puzzleMain
 jsr puzzleStart                    ;Alles auf Anfang stellen
 
;*******************************************************************************
;*** Eingabeschleife für das Puzzle
;*******************************************************************************
puzzleMainLoop                      ;Start der Endlosschleife
 jsr checkInput                     ;Joysticks prüfen & Eingaben verarbeiten
 jsr drawTimer                      ;ggf. den Zeitbalken aktualisieren
 jmp puzzleMainLoop                 ;Endlosschleife



;*******************************************************************************
;*** Ein neues Puzzle beginnen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: Alle veränderbaren Werte wurden zurückgestellt
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone puzzleStart
puzzleStart
 lda #COLOR_BLACK                   ;Schwarz in den Akku und
 sta VIC_BACKGROUNDCOLOR            ;Hintergrund- sowie
 sta VIC_BORDERCOLOR                ;Rahmenfarbe setzen
 ;*** Puzzle-BS anzeigen
 ldx #<screenPuzzle                 ;LSB ins X-Register
 ldy #>screenPuzzle                 ;MSB ins Y-Register
 jsr drawScreen                     ;Bild ausgeben
 lda #$ff                           ;leeres Feld wieder
 sta tileOrder+8                    ;rechts unten
 lda #$08                           ;auch die Hilfsvariable
 sta freetilePos                    ;zurückstellen
 ldx #$07                           ;Ein gelöstest Puzzle erstellen
.loop
 txa
 sta tileOrder,X
 dex
 bpl .loop
 jsr loadSprites                    ;Sprites laden
 jsr shuffleTiles                   ;Puzzle mischen
 lda #$14                           ;Timer zurückstellen
 sta timer
 lda #$00                           ;Uhr auf NULL stellen
 sta CIA1_CLOCK_SECONDS             ;Sekunden und
 sta CIA1_CLOCK_TENTH               ;zehntel reichen uns
 rts                                ;zurück



;*******************************************************************************
;*** Der Spieler hat gewonnen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone puzzleSolved
puzzleSolved
 lda #$00                           ;abschalten aller
 sta VIC_SPRITEACTIVE               ;Sprites
 ;*** 'Geschaff'-BS anzeigen
 ldx #<screenSolved                 ;LSB ins X-Register
 ldy #>screenSolved                 ;MSB ins Y-Register
 jsr drawScreen                     ;Bild ausgeben
.loop
 jsr joystickInput                  ;prüfen, ob der
 and #JOY_FIRE                      ;Feuerknopf gedrückt wurde
 bne .loop
.loop1
 jsr joystickInput                  ;prüfen, ob der
 and #JOY_FIRE                      ;Feuerknopf wieder losgelassen wurde
 beq .loop1
 jmp main                           ;Noch gibt es keinen weiteren Level,
                                    ;also erstmal einfach neu starten.



;*******************************************************************************
;*** Der Spieler hat verloren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone gameOver
gameOver
 lda #$00                           ;Alle Sprites
 sta VIC_SPRITEACTIVE               ;abschalten
 ;*** GameOver-BS anzeigen
 ldx #<screenGameOver               ;LSB ins X-Register
 ldy #>screenGameOver               ;MSB ins Y-Register
 jsr drawScreen                     ;Bild ausgeben
.loop
 jsr joystickInput                  ;warten bis
 and #JOY_FIRE                      ;der Feuerknopf gedrückt wurde
 bne .loop
.loop1
 jsr joystickInput                  ;und jetzt warten bis
 and #JOY_FIRE                      ;der Feuerknopf wieder losgelassen wurde
 beq .loop1
 jmp main                           ;Programm neu starten



;*******************************************************************************
;*** Zeitbalken zeichnen, wenn timerStep * 1 Sekunde abgelaufen sind
;*** und das Zeitintervall timer herunterzählen.
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone drawTimer
drawTimer
 lda CIA1_CLOCK_SECONDS             ;Sekunden in den Akku
 cmp timerStep                      ;ist unser Zeitintervall verstrichen?
 bne .exit                          ;falls nicht, wieder zurück
 lda #<(COLORRAMADR+32)             ;Startadresse in die Zero-Page
 sta ZP_COLORRAMADR
 lda #>(COLORRAMADR+32)
 sta ZP_COLORRAMADR+1
 lda #$14                           ;Unser Zeitbalken ist 20 Einheiten hoch
 sec                                ;Carry für SBC setzen
 sbc timer                          ;akt. Timer abziehen
 tax                                ;und nach X kopieren
 dec timer                          ;aktuellen Timer herunterzählen
 bmi gameOver                       ;wenn die Zeit um ist -> GAME OVER
.loop
 lda ZP_COLORRAMADR                 ;Startadresse fürs Ändern der jeweiligen
 clc                                ;Zeile (neue Farbe) errechnen
 adc #$28                           ;Dazu addieren wir für jedes verstrichene
 sta ZP_COLORRAMADR                 ;Zeitinterval 40 auf die Zero-Page-Adresse
 lda ZP_COLORRAMADR+1               ;um eine Zeile nach unten zu wandern.
 adc #$00                           ;ggf. auch das MSB korrigieren
 sta ZP_COLORRAMADR+1
 dex                                ;Schleifenzähler verringern
 bpl .loop                          ;und weiter addieren, solange positiv
 lda COLOR_DARKGREY                 ;Neue Farbe in den Akku
 ldy #$03                           ;der Balken ist max. vier Zeichen breit
.loop1
 sta (ZP_COLORRAMADR),Y             ;Farbe setzen
 dey                                ;nächstes Zeichen
 bpl .loop1                         ;solange Y positiv ist
 lda #$00                           ;nur die Sekunden 
 sta CIA1_CLOCK_SECONDS             ;wieder auf Null stellen
.exit
 rts                                ;zurück



;*******************************************************************************
;*** Prüfen, ob das Puzzle gelöst wurde
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: A = 0 (NICHT gelöst)
;***           A = 1 (gelöst)
;*******************************************************************************
;*** ändert  : A, X, SR
;*******************************************************************************
!zone checkPuzzle
checkPuzzle
 ldx tileOrder+8                    ;steht das leere Feld rechts unten?
 bpl .exit                          ;falls nein zum Exit springen
 ldx #$07                           ;sonst die restlichen Felder prüfen
.loop
 txa                                ;X für Vergleich in den Akku
 cmp tileOrder,X                    ;Akku mit Puzzle vergleichen
 bne .exit                          ;wenn NICHT identisch zum EXIT
 dex                                ;nächstes Puzzleteil
 bne .loop                          ;prüfen, solange > 0
 lda #$01                           ;hier ist alles OK, also 1 in den Akku
 rts                                ;zurück
.exit
 lda #$00                           ;nicht gelöst, 0 in den Akku
 rts                                ;zurück



;*******************************************************************************
;*** Joystickeingabe prüfen
;*** Prüft ob nur eine Richtung gedrückt ist und die Richtung aktuell erlaubt
;*** ist. Wenn beides zutrifft, wird die Eingabe über perfomInput verarbeitet.
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: letzte Eingabe in inputState
;*******************************************************************************
;*** ändert  : A, X, Y, SR, inputState
;*******************************************************************************
!zone checkInput
checkInput
 ldx inputState                     ;letzte Eingabe ins X-Register
 jsr joystickInput                  ;Aktuellen Joystick-Status holen
 and #%00011111                     ;Nur die unteren 5-BIT sind von Interesse
 eor #%00011111                     ;Da Low-Aktiv, umkehren
 beq .exit                          ;keine Eingabe, dann ENDE
 sta inputState                     ;sonst merken wir uns den angepassten Wert
 cpx inputState                     ;hat sich die Eingabe geändert?
 beq .exit                          ;wenn, nein -> ENDE
.loop                               ;prüfen, ob nur eine Richtung gedrückt ist
 lsr                                ;solange nach rechts verschieben, bis C=1
 bcc .loop                          ;wenn C noch 0, weiter verschieben
 bne .exit                          ;sonst ENDE, falls Rest <> 0
 ldx freetilePos                    ;Pos des freien Feldes ins X-Reg.
 lda inputHelper,X                  ;erlaubte Richtungen in den Akku
 and inputState                     ;mit unserer aktuellen Eingabe prüfen 
 beq .exit                          ;wenn die Richtung verboten ist, ENDE
 jsr performInput                   ;hier ist alles OK, also verarbeiten
 jsr checkPuzzle                    ;nach jedem Zug prüfen, ob es gelöst wurde
 beq .exit                          ;steht eine 0 im Akku, dann nicht gelöst
 jmp puzzleSolved                   ;Das Puzzle wurde gelöst!
.exit
 rts                                ;zurück



;*******************************************************************************
;*** Ton fürs Bewegen eines Puzzleteils
;*******************************************************************************
;*** Übergabe: X = Lautstärke
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone playTileSound
playTileSound
 lda #$00                           ;Anschlag & Abschwellen auf 0
 sta SID_SUSTAIN_RELEASE
 lda #SID_WAVE_WHITENOISE           ;Rausen
 sta SID_VOICE1_WAVEFORM            ;für die 1. Stimme
 lda #$05                           ;Niedrige Frequenz                          
 sta SID_VOICE1_FREQ_MSB            ;für die erste Stimme
 stx SID_VOLUME                     ;Lautstärke laut X-Register
 rts                                ;zurück



;*******************************************************************************
;*** Joystickeingabe verarbeiten
;*** Vertauscht das freie Feld und das entsprechende Puzzleteil.
;*** Anschließend wird alles erneut gezeichnet.
;*******************************************************************************
;*** Übergabe: Akku = korrigierte Eingabe
;*******************************************************************************
;*** Rückgabe: Getauschte Puzzleteile in tileOrder
;*******************************************************************************
;*** ändert  : A, X, Y, SR, freetilePos, tileOrder
;*******************************************************************************
!zone performInput
performInput
 lsr                                ;Per Bitverschiebung die Eingabe suchen
 bcc .down                          ;wenn C=0, dann runter prüfen
 ;*** Joystick rauf
 lda freetilePos                    ;akt. Pos. des leeren Feldes in den Akku
 clc                                ;C=0 für ADC
 adc #$03                           ;+3, für das Feld unter dem freien 
 tay                                ;im Y-Reg. merken
 lda tileOrder,Y                    ;Puzzleteil in den Akku
 tax                                ;im X-Reg. merken
 lda #$ff                           ;Wert für freies Feld in den Akku
 sta tileOrder,Y                    ;und an errechneter Pos. speichern
 txa                                ;gemerktes Puzzleteil in den Akku
 sta tileOrder-3,Y                  ;und ins bisher freie Feld kopieren
 sty freetilePos                    ;Pos. des leeren Feldes nach freetilepos:
 jmp .refresh                       ;und alles neu zeichnen
.down                               ;hier ist fast alles wie eben bei Jostick rauf
 lsr
 bcc .left                          ;nur dass wir ggf. 'nach links' prüfen
 ;*** Joystick runter
 lda freetilePos
 sec
 sbc #$03                           ;und jetzt -3 rechnen, für das über dem
 tay                                ;leeren Feld liegende Puzzleteil
 lda tileOrder,Y
 tax
 lda #$ff
 sta tileOrder,Y
 txa
 sta tileOrder+3,Y                  ;auch hier das andere Vorzeichen beachten
 sty freetilePos
 jmp .refresh
.left
 lsr
 bcc .right
 ;*** Joystick links    
 ldy freetilePos                    ;hier vertauschen wir einfach das leere
 iny                                ;Feld mit dem Teil rechts
 lda tileOrder,Y
 tax
 lda #$ff
 sta tileOrder,Y
 txa
 sta tileOrder-1,Y
 sty freetilePos
 jmp .refresh
.right                              ;wenn es keine der anderen Richtungen war,
 ;*** Joystick rechts               ;dann bleibt nur rechts übrig
 ldy freetilePos                    ;das freie Feld mit dem Teil links tauschen
 dey                               
 lda tileOrder,Y
 tax
 lda #$ff
 sta tileOrder,Y
 txa
 sta tileOrder+1,Y
 sty freetilePos
.refresh
 ldx #$0f                           ;max. Lautstärke
 jsr playTileSound                  ;Ton wiedergeben
 jsr spriteSetPositions             ;Sprites positionieren
 jsr drawTiles                      ;Hintergründe zeichnen
 lda #$00                           ;Lautstärke auf
 sta SID_VOLUME                     ;0 setzen!
 rts                                ;zurück



;*******************************************************************************
;*** Sprites initialisieren
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone loadSprites
loadSprites
 ;*** 64-Byte-Block berechnen
 lda #<spriteTiles                  ;LSB der Spritedaten holen
 sta calc16Bit                      ;im 'Hilfsregister' speichern
 lda #>spriteTiles                  ;MSB auch
 sta calc16Bit+1                    ;ins 'Hilfregister'
 ldx #$02                           ;Schleifenzähler

.loop
 asl calc16Bit                      ;LSB nach links 'shiften'
 rol calc16Bit+1                    ;MSB nach links 'rotieren', wg. Carry-Flag!
 dex                                ;Schleifenzähler verringern
 bne .loop                          ;wenn nicht 0, nochmal
 ldy #$00
 lda calc16Bit+1                    ;Im MSB des 'Hilfsregisters' steht der 64-Byte-Block

 ;*** Start Block für alle Sprites setzen
.next
 sta SPRITEPOINTER0,Y               ;In der zuständigen Speicherstelle ablegen
 clc
 adc #$01                           ;Das nächste Sprite folgt direkt dahinter, also +1
 iny
 cpy #$08
 bne .next
 
 jsr spriteSetPositions
 jsr drawTiles                      ;Hintergründe zeichnen
 
 ;*** Hauptfarbe für alle Sprites auf blau setzen
 lda #COLOR_BLUE
 sta VIC_SPRITE0COLOR
 sta VIC_SPRITE1COLOR
 sta VIC_SPRITE2COLOR
 sta VIC_SPRITE3COLOR
 sta VIC_SPRITE4COLOR
 sta VIC_SPRITE5COLOR
 sta VIC_SPRITE6COLOR
 sta VIC_SPRITE7COLOR

 ;*** Multicolor-Farben setzen
 lda #COLOR_GREEN
 sta VIC_SPRITEMULTICOLOR1
 lda #COLOR_RED
 sta VIC_SPRITEMULTICOLOR0
 
 ;*** Die Sprites sollen VOR dem Hintergrund erscheinen
 lda #%00000000
 sta VIC_SPRITEDEEP

 ;*** ALLE Sprites in Breite & Höhe verdoppeln,
 ;*** auf Multicolor setzen und aktivieren.
 lda #%11111111
 sta VIC_SPRITEDOUBLEWIDTH
 sta VIC_SPRITEDOUBLEHEIGHT
 sta VIC_SPRITEMULTICOLOR
 sta VIC_SPRITEACTIVE
 rts                               ;Zurück



;*******************************************************************************
;*** Positioniert alle Sprites
;*******************************************************************************
;*** Übergabe: Bei tileOrder 'steht', auf welchem Feld, welches Puzzleteil
;***           liegt. Da Puzzleteil = Sprite gilt, finden wir so auch das 
;***           jeweilige Sprite.
;*******************************************************************************
;*** Rückgabe: Im jeweiligen SPRITE(n)X & SPRITE(n)Y Register stehen die 
;***           richtigen Koordinaten
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone spriteSetPositions
spriteSetPositions
 ldx #$08                          ;max. 9 Felder
.loop
 ldy tileOrder,X                   ;was befindet sich auf dem Feld?
 bmi .skip                         ;wenn negativ, dann leeres Feld überspringen
 txa                               ;da wir X verändert -> Akku
 pha                               ;und auf dem Stack merken
 tya                               ;Y muss mit 2 multipliziert werden -> Akku
 asl                               ;Akku * 2
 tay                               ;und zurück nach Y
 lda spritePosHelper,X             ;Hilfswert in den Akku holen
 tax                               ;und nach X kopieren
 lda spritePos,X                   ;X-Position fürs aktuelle Feld holen
 sta VIC_SPRITE0X,Y                ;und im richtigen Sprite eintragen
 lda spritePos+1,X                 ;Y-Position fürs aktuelle Feld holen
 sta VIC_SPRITE0Y,Y                ;und im Sprite speichern
 pla                               ;alten X-Wert vom Stack in den Akku
 tax                               ;und Akku nach X kopieren
.skip                              ;Ziel, falls leeres Feld
 dex                               ;nächstes Feld
 bpl .loop                         ;solange positiv nach .loop
 rts                               ;zurück

 
 
;*******************************************************************************
;*** Hintergründe der einzelnen Puzzleteile zeichnen
;*** gerade Sprite-Nr.   = heller Hintergrund
;*** ungerade Sprite-Nr. = dunkler Hintergrund
;*** $ff                 = freies Feld
;*******************************************************************************
;*** Übergabe: Abhängig von tileorder:
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone drawTiles
drawTiles
 lda #<(SCREENRAMADR+(15*40+15))    ;ZP-Adresse für den BS-Speicher
 sta ZP_SCREENRAMADR                ;auf die linke obere Ecke
 lda #>(SCREENRAMADR+(15*40+15))    ;des 9. Feldes setzen
 sta ZP_SCREENRAMADR+1
 lda #<(COLORRAMADR+(15*40+15))     ;ZP-Adresse für das Color-RAM
 sta ZP_COLORRAMADR                 ;auf die linke obere Ecke
 lda #>(COLORRAMADR+(15*40+15))     ;des 9. Feldes setzen
 sta ZP_COLORRAMADR+1

 ldx #$08                           ;max. 9 Felder
.loop
 txa                                ;X-Register auf dem
 pha                                ;Stack merken
 ldy #COLOR_GREY                    ;Erstmal grau ins Y-Register
 lda tileOrder,X                    ;was steht im Feld -> Akku
 bmi .skip                          ;wenn freies Feld -> .skip
 and #%00000001                     ;gerades oder ungerades Sprite?
 bne .skip1                         ;wenn ungerade, Farbe behalten -> .skip1
 ldy #COLOR_LIGHTGREY               ;Hintergrund für gerades Sprite
.skip1
 lda #160                           ;invertiertes Leerzeichen in den Akku
 jmp .skip2
.skip                               ;freies Feld
 lda #102                           ;Karriertes Zeichen in den Akku
 ldy #COLOR_DARKGREY
.skip2
 sta ZP_LOOPHELPER                  ;Zeichen auf die Zero-Page
 sty ZP_LOOPHELPER+1                ;Farbe auf die Zero-Page

 ldx #$04                           ;Zeile ins X-Register
.loop1
 ldy #$04                           ;Spalte in Y-Register
.loop2
 lda ZP_LOOPHELPER                  ;Zeichen holen
 sta (ZP_SCREENRAMADR),Y            ;ab in den BS-Speicher
 lda ZP_LOOPHELPER+1                ;Farbe holen
 sta (ZP_COLORRAMADR),Y             ;und ins Farb-RAM
 dey                                ;Spalte verringern
 bpl .loop2                         ;solange positiv nächstes Zeichen ausgeben

 clc                     
 lda ZP_SCREENRAMADR                ;LSB der BS-Speicherpostion holen
 adc #$28                           ;eine Zeile (40 BYTES) addieren
 sta ZP_SCREENRAMADR                ;und wieder speichern
 bcc .skip3                         ;wenn kein Carry weiter springen
 inc ZP_SCREENRAMADR+1              ;sonst MSB erhöhen 

.skip3                              ;das Gleiche wie eben, nur fürs Farb-RAM
 clc
 lda ZP_COLORRAMADR
 adc #$28
 sta ZP_COLORRAMADR
 bcc .skip4
 inc ZP_COLORRAMADR+1
.skip4
 dex                                ;Zeilenzähler verringern
 bpl .loop1                         ;solange positiv, nächste Zeile

 sec
 lda ZP_SCREENRAMADR                ;nun die Startposition vom Feld
 sbc #$CD                           ;links berechnen, dazu 205 abziehen
 sta ZP_SCREENRAMADR                ;da wir eben bereits 40 addiert haben
 bcs .skip5
 dec ZP_SCREENRAMADR+1
.skip5                              ;auch fürs Farb-RAM
 sec
 lda ZP_COLORRAMADR
 sbc #$CD
 sta ZP_COLORRAMADR
 bcs .skip6
 dec ZP_COLORRAMADR+1

.skip6
 pla                                ;Stack wieder zurück
 tax                                ;ins X-Register
 cmp #$06                           ;untere 3 Felder fertig?
 beq .rowUp                         ;falls ja, einen Block nach oben
 cmp #$03                           ;mittlere drei Felder fertig?
 bne .skip7                         ;falls nein, nächstes Feld
 jmp .rowUp                         ;sonst einen Block nach oben
.skip7
 dex                                ;X (jetzt wieder Feld-Nr.) verringern
 bpl .loop                          ;schon alle 9 Felder verarbeitet?
 rts                                ;zurück

.rowUp
 sec
 lda ZP_SCREENRAMADR                ;immer wenn drei Felder gezeichnet wurden,
 sbc #$b9                           ;müssen wir einen Block nach oben, dazu 
 sta ZP_SCREENRAMADR                ;185 abziehen 
 bcs .skip8                         ;Achtung oben wurden bereits 205 subtrahiert!
 dec ZP_SCREENRAMADR+1
.skip8                              ;auch hier wieder das gleiche fürs Farb-RAM
 sec
 lda ZP_COLORRAMADR
 sbc #$b9
 sta ZP_COLORRAMADR
 bcs .skip7
 dec ZP_COLORRAMADR+1
 jmp .skip7                         ;weiter mit dem nächsten Feld



;*******************************************************************************
;*** Die Puzzleteile zufällig verteilen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: Neue Position der Puzzleteile in der Tabelle tileOrder
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone shuffleTiles
shuffleTiles
 ldx #$20                           ;Anzahl der Schritte, je höher je schwerer
.loop
 txa                                ;X auf den 
 pha                                ;Stack
.loop1
 jsr rndTimer                       ;Zufallszahl holen
 and #%00000011                     ;wir brauchen nur die beiden unteren BITs
 tay                                ;ins Y-Register
 lda #JOY_FIRE                      ;nur BIT 4 im Akku setzen 
.loop2
 lsr                                ;Akku bitweise nach rechts
 dey                                ;Y verringern
 bpl .loop2                         ;solange positiv, weiter verschieben
 ldx freetilePos                    ;Pos des freien Feldes ins X-Reg.
 and inputHelper,X                  ;erlaubte Richtungen prüfen
 beq .loop1                         ;wenn die Richtung verboten ist, nochmal
 jsr performInput                   ;sonst, Eingabe verarbeiten
 pla                                ;gemerktes X
 tax                                ;wieder vom Stack holen
 dex                                ;verringern
 bne .loop                          ;und ggf. weiter 'verwürfeln'
 jsr checkPuzzle                    ;prüfen ob das Puzzle zufällig gelöst wurde
 bne shuffleTiles                   ;falls gelöst, nochmal von vorne
 rts                                ;zurück



;*******************************************************************************
;*** Zufallszahl per Timer-Verknüpfungen bilden.
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: Zufallszahl im Akku
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone rndTimer
rndTimer
 lda $dc04                          ;Low-Byte  von Timer A aus dem CIA-1
 eor $dc05                          ;High-Byte von Timer A aus dem CIA-1
 eor $dd04                          ;Low-Byte  von Timer A aus dem CIA-2
 adc $dd05                          ;High-Byte von Timer A aus dem CIA-2
 eor $dd06                          ;Low-Byte  von Timer B aus dem CIA-2
 eor $dd07                          ;High-Byte von Timer B aus dem CIA-2
 eor rndSeed                        ;noch mit unserem Quellwert verknüpfen
 sta rndSeed                        ;und Rückgabe als neue Quelle speichern
 rts                                ;zurück



;*******************************************************************************
;*** Titel-Bildschirm anzeigen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: -
;*******************************************************************************
;*** ändert  : A, X, Y, SR
;*******************************************************************************
!zone showScreen_Title
showScreen_Title
 lda #COLOR_BLACK                   ;Schwarz in den Akku und
 sta VIC_BORDERCOLOR                ;als Rahmen- und
 sta VIC_BACKGROUNDCOLOR            ;Hintergrundfarbe setzen
 lda #INPUT_NONE                    ;Aktuelles Eingabegerät auf unbekannt
 sta inputDevice                    ;setzen

 ;*** StartBS anzeigen
 ldx #<screenTitle                  ;LSB des Titelbildes ins X-Register
 ldy #>screenTitle                  ;MSB des Titelbildes ins Y-Register
 jsr drawScreen                     ;Bildausgeben

.wait
 jsr selectInputDevice              ;Auf Eingabegerät prüfen
 lda inputDevice                    ;Aktuelles Gerät in den Akku
 beq .wait                          ;solange keins gewählt wurde, warten
 rts
 
 

;*******************************************************************************
;*** Den ausgewählten Joystick abfragen
;*******************************************************************************
;*** Übergabe: CIA1_<PORT> an joystickInput+1
;*******************************************************************************
;*** Rückgabe: A = aktueller Status
;***           inputState: = aktueller Status
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone joystickInput
joystickInput
 lda CIA1_DATAPORT_A                ;Aktuellen Status von Joy-1 oder 2 holen
                                    ;Nach der Joystick-Wahl speichern wir
                                    ;die richtige Adresse direkt hinterm lda
 sta inputState                     ;Aktuellen Status merken
 rts                                ;zurück
 
 
 
;*******************************************************************************
;*** Eingabegerät wählen
;*******************************************************************************
;*** Übergabe: -
;*******************************************************************************
;*** Rückgabe: Gewähltes 'Gerät' in inputDevice
;*******************************************************************************
;*** ändert  : A, SR
;*******************************************************************************
!zone selectInputDevice
selectInputDevice
 lda #<CIA1_DATAPORT_A              ;Das LSB von CIA1_DATAPORT_A (Port-2) hinter                                    
 sta joystickInput+1                ;den lda-Befehl bei joystickInput ablegen
 jsr joystickInput                  ;und auf Eingabe prüfen
 and #JOY_FIRE                      ;Ist aktuell der Feuerknopf gedrückt?
 bne .joy1                          ;wenn nicht, weiter mit Port-1
 lda #INPUT_JOY2                    ;sonst, haben wir unseren Joystick
 sta inputDevice                    ;und merken es in inputDevice
.wait
 jsr joystickInput:                 ;weiter den Joystick-2 prüfen
 and #JOY_FIRE                      ;ist immer noch Feuer gedrückt?
 beq .wait                          ;wenn ja, warte bis losgelassen
 rts                                ;und zurück

.joy1                               ;Wie eben, nur für den Joystick an Port-1
!zone joy1
 inc joystickInput+1                ;um CIA1_B zu verwenden reicht ein inc 
 jsr joystickInput
 and #JOY_FIRE
 bne .exit     
 lda #INPUT_JOY1
 sta inputDevice
.wait
 jsr joystickInput
 and #JOY_FIRE
 beq .wait
.exit
 rts                                ;zurück



;Allgemeine Funktionen einbinden
!source "..\_Includes_\love_lib.asm"



;*******************************************************************************
;*** Daten
;*******************************************************************************
!zone Data
;*** gewähltes Eingabegerät:
;*** $00 = noch nicht gewählt
;*** $01 = Joy-1
;*** $02 = Joy-2
inputDevice
 !byte $00

;*** Aktueller Status
inputState
 !byte $00

;*** Was befindet sich auf dem jeweiligen Feld?
;*** 0-7: Sprite-Nr. = Puzzleteil
;*** $ff: leeres Feld
tileOrder
 ;Feld   1    2    3  -------
 !byte $00, $01, $02 ;|1|2|3|
 ;Feld   4    5    6  -------
 !byte $03, $04, $05 ;|4|5|6|
 ;Feld   7    8    9  -------
 !byte $06, $07, $ff ;|7|8|9|
                     ;-------
 
;Hilfstabelle für die Spritepositionen
spritePosHelper
 !for I = 0 TO 8
    !byte I*2
 !end

spritePos
 ;*** Spriteposition für jedes Feld (s. o.)
 !byte $40, $5a                     ;Feld 1  (X, Y)
 !byte $68, $5a                     ;Feld 2  (X, Y)
 !byte $90, $5a
 !byte $40, $82
 !byte $68, $82                     ;...
 !byte $90, $82
 !byte $40, $aa
 !byte $68, $aa                     ;Feld 8  (X, Y)
 !byte $90, $aa                     ;Feld 9  (X, Y)

;*** Hilfsregsiter für 16-Bit-Operationen
calc16Bit
 !word $0000

;*** Restzeit, bis zur Niederlage
timer
 !byte $14

;*** Timerintervall
timerStep
 !byte $02

;*** Basis für die Zufallszahlen
rndSeed
 !byte $00

;*** Wo befindet sich das leere Feld
freetilePos
 !byte $08

;*** Hilfstabelle erlaubte Richtungen je Position
inputHelper
 !byte %00000101, %00001101, %00001001
 !byte %00000111, %00001111, %00001011
 !byte %00000110, %00001110, %00001010

;*** unsere Puzzleteile
!align 63,0
spriteTiles
 !media "Tiles.spriteproject",sprite,0,8
 
;*** unser Titelbild
screenTitle
 !media "Title.charscreen",charcolor
 
;*** unser Hintergrundbild für das Puzzle
screenPuzzle
 !media "Background.charscreen",charcolor

;*** Verloren: Die Zeit ist abgelaufen
screenGameOver
 !media "GameOver.charscreen",charcolor

;*** Geschafft, das Puzzle wurde gelöst
screenSolved
 !media "Solved.charscreen",charcolor